@@ -6,14 +6,22 @@
  * mpage:    a program to reduce pages of print so that several pages
  *           of output appear on one printed page.
  *
- * Copyright (c) 1994-1999 Marcel J.E. Mol, The Netherlands
+ * Copyright (c) 1994-2004 Marcel J.E. Mol, The Netherlands
  * Copyright (c) 1988 Mark P. Hahn, Herndon, Virginia
  *  
- *     Permission is granted to anyone to make or distribute verbatim
- *     copies of this document as received, in any medium, provided
- *     that this copyright notice is preserved, and that the
- *     distributor grants the recipient permission for further
- *     redistribution as permitted by this notice.
+ *     This program is free software; you can redistribute it and/or
+ *     modify it under the terms of the GNU General Public License
+ *     as published by the Free Software Foundation; either version 2
+ *     of the License, or (at your option) any later version.
+ *
+ *     This program is distributed in the hope that it will be useful,
+ *     but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *     GNU General Public License for more details.
+ *
+ *     You should have received a copy of the GNU General Public License
+ *     along with this program; if not, write to the Free Software
+ *     Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  *
  */
 
@@ -33,6 +41,7 @@ do_args(int argc, char **argv, int envflag)
 {
     char *optstr;
     int consumed;
+    int finished;
     int currarg;
     int opterrors;
 
@@ -46,10 +55,16 @@ do_args(int argc, char **argv, int envflag)
     }
 
 #define OPTARG()  \
-    { consumed = 1; if(*++optstr == '\0') optstr = argv[++currarg]; }
+    { char opt = *optstr; \
+      consumed = 1; if(*++optstr == '\0') optstr = argv[++currarg]; \
+      if (!optstr) { fprintf (stderr, "%s: -%c: option requires argument\n", \
+                    MPAGE, opt); return -1; } \
+    }
+
 
     opterrors = 0;
-    for (currarg = 1; currarg < argc; currarg++) {
+    finished = 0;
+    for (currarg = 1; !finished && currarg < argc; currarg++) {
         if (*argv[currarg] != '-') {
             if (envflag)
                 opterrors++;
@@ -64,6 +79,9 @@ do_args(int argc, char **argv, int envflag)
                                         MPAGE, *optstr);
                         opterrors++;
                         break;
+                case '-':
+                        finished = 1;
+                        break;
                 case '1':
                         sheetindex = 0;
                         break;
@@ -81,7 +99,7 @@ do_args(int argc, char **argv, int envflag)
                                      UPDOWN : LEFTRIGHT;
                         break;
                 case 'A':    /* A4 sized, european paper */
-                        opt_page = "A4";
+                        opt_page = "A4"; /* deprecated */
                         break;
                 case 'b':     /* base paper type */
                         OPTARG();
@@ -221,6 +239,10 @@ do_args(int argc, char **argv, int envflag)
                             opt_alt[opt_jarg] = atoi(++optstr);
                         opt_jarg++;
                         break;
+                case 'J':
+                        OPTARG();
+                        ps_pagenum = atoi(optstr) - 1;
+                        break;
                 case 'k':    /* toggle kill on trailer */
                         opt_killtrail = 1 - opt_killtrail;
                         break;
@@ -378,12 +400,19 @@ do_args(int argc, char **argv, int envflag)
                 case 'T':
                         opt_tumble = 1 - opt_tumble;
                         break;
+                case 'u':
+                        check_utf8 = 1 - check_utf8;
+                        break;
                 case 'U':    /* Letter sized, US paper */
-                        opt_page = "Letter";
+                        opt_page = "Letter"; /* deprecated */
                         break;
                 case 'v':    /* verbose (print page count) */
                         opt_verbose = 1 - opt_verbose;
                         break;
+                case 'V':    /* Version */
+                        fprintf(stderr, "mpage version %s\n", VERSION);
+                        exit(0);
+                        break;
                 case 'W':
                         OPTARG();
                         opt_width = atoi(optstr);
@@ -442,7 +471,6 @@ do_env()
         libdir = env;
 
     if ((env = getenv("MPAGE")) != NULL) {
-        strcpy(copy, env);
         if ((copy = strdup(env)) == NULL) {
             fprintf(stderr, "%s: Unable to alloc memory for environment args\n",
                     MPAGE);
