@@ -6,14 +6,22 @@
  * mpage:	a program to reduce pages of print so that several pages
  * 	  	of output appear on one printed page.
  *
- * Copyright (c) 1994-1999 Marcel J.E. Mol, The Netherlands
+ * Copyright (c) 1994-2004 Marcel J.E. Mol, The Netherlands
  * Copyright (c) 1988 Mark P. Hahn, Herndon, Virginia
  *  
- *     Permission is granted to anyone to make or distribute verbatim
- *     copies of this document as received, in any medium, provided
- *     that this copyright notice is preserved, and that the
- *     distributor grants the recipient permission for further
- *     redistribution as permitted by this notice.
+ *     This program is free software; you can redistribute it and/or
+ *     modify it under the terms of the GNU General Public License
+ *     as published by the Free Software Foundation; either version 2
+ *     of the License, or (at your option) any later version.
+ *
+ *     This program is distributed in the hope that it will be useful,
+ *     but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *     GNU General Public License for more details.
+ *
+ *     You should have received a copy of the GNU General Public License
+ *     along with this program; if not, write to the Free Software
+ *     Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  *
  */
 
@@ -21,6 +29,39 @@
 #include "mpage.h"
 #include "string.h"
 
+#define PAPERSIZE_FILE  "/etc/papersize"
+void
+check_papersize()
+{
+    FILE * psfp;
+    char buf[LINESIZE];
+    
+    if (access(PAPERSIZE_FILE, R_OK))
+        return; /* no access or no papersize file. Just do nothing */
+
+    if ((psfp = fopen(PAPERSIZE_FILE, "r")) == NULL)
+        return; /* no access or no papersize file. Just do nothing */
+
+    while (fgets(buf, LINESIZE, psfp) != NULL) {
+        int i = 0;
+        int j = strlen(buf);
+
+        while (i < j && (buf[i] == ' ' || buf[i] == '\t'))
+            i++; /* skip leading space */
+        while (--j >= 0 && (buf[j] == ' ' || buf[j] == '\t' || buf[j] == '\n' || buf[j] == '\r'))
+            buf[j] = '\0'; /* skip trailing space */
+
+        if (strlen(buf + i) && buf[i] != '#' ) {
+            select_pagetype(buf + i);
+            break;
+        }
+    }
+
+    fclose (psfp);
+
+    return;
+
+} /* check_papersize */
 
 
 void
@@ -103,8 +144,22 @@ show_pagetypes()
 int
 xbase1()
 {
+#if 1
+        int val;
+
+        if (opt_duplex && ((ps_pagenum % 2) == 0))
+                val = sheetmargin_right + sheetheader_right;
+        else
+                val = sheetmargin_left + sheetheader_left;
+
+        Debug(DB_POINTS, "%%xbase1: %d\n", val);
+
+        return val;
+#else
+
 	Debug(DB_POINTS, "%%xbase1: %d\n", sheetmargin_left + sheetheader_left);
 	return sheetmargin_left + sheetheader_left;
+#endif
 } /* xbase1 */
 
 
@@ -112,6 +167,21 @@ xbase1()
 int
 xbase2()
 {
+#if 1
+        int val;
+
+        if (opt_duplex && ((ps_pagenum % 2) == 0))
+                val = (ps_width - sheetmargin_left - sheetmargin_right
+                          - sheetheader_left - sheetheader_right) / 2 +
+                sheetmargin_right + sheetheader_right;
+        else
+                val = (ps_width - sheetmargin_left - sheetmargin_right
+                          - sheetheader_left - sheetheader_right) / 2 +
+                sheetmargin_left + sheetheader_left;
+
+        Debug(DB_POINTS, "%%xbase2: %d\n", val);
+        return  (val);
+#else
 	Debug(DB_POINTS, "%%xbase2: %d\n",
                 (ps_width - sheetmargin_left - sheetmargin_right
                           - sheetheader_left - sheetheader_right) / 2 +
@@ -119,6 +189,7 @@ xbase2()
         return  (ps_width - sheetmargin_left - sheetmargin_right
                           - sheetheader_left - sheetheader_right) / 2 +
                 sheetmargin_left + sheetheader_left;
+#endif
 } /* xbase2 */
 
 
@@ -367,54 +438,50 @@ sheetheader(outfd, fname)
         switch (orientation) {
             case PORTRAIT:
                 sheetheader_top = 14;
-                if (*sheethead != '\0') {
-                    fprintf(outfd,
-                           "%d %d (%s) stringwidth pop sub 2 div add %d moveto",
-                            xbase1(), xwid1(), sheethead,
-                            ytop4() + fsize / 4);
-                    fprintf(outfd, " (%s) show\n", sheethead);
-                }
-                else {
-                    /*
-                     * Filename on the left
-                     */
-                    fprintf(outfd, "%d %d moveto (%s) show\n",
-                            xbase1() + 3, ytop4() + fsize / 4, fname);
-                    /*
-                     * Pagenumber on the right
-                     */
-                    fprintf(outfd,
-                            "%d (Page %d) stringwidth pop sub 3 sub %d moveto",
+#if 0
+		/* centered text... */
+                fprintf(outfd, "%d %d (%s) stringwidth pop sub 2 div add %d moveto",
+                               xbase1(), xwid1(), sheethead,
+                               ytop4() + fsize / 4);
+                fprintf(outfd, " (%s) show\n", sheethead);
+#endif
+                /*
+                 * Filename/Header on the left
+                 */
+                fprintf(outfd, "%d %d moveto (%s) show\n",
+                            xbase1() + 3, ytop4() + fsize / 4,
+			    *sheethead != '\0' ? sheethead : fname);
+                /*
+                 * Pagenumber on the right
+                 */
+                fprintf(outfd, "%d (Page %d) stringwidth pop sub 3 sub %d moveto",
                             xbase1() + xwid1(), ps_pagenum,
                             ytop4() + fsize / 4);
-                    fprintf(outfd, " (Page %d) show\n", ps_pagenum);
-                }
+                fprintf(outfd, " (Page %d) show\n", ps_pagenum);
                 break;
             case LANDSCAPE:
             case LANDSCAPE_PORTRAIT:
                 sheetheader_right = 14;
-                if (*sheethead != '\0') {
-                    fprintf(outfd,
-                           "%d %d (%s) stringwidth pop 2 div add moveto",
-                            xbase1() + xwid1() + fsize/4,
-                            ybase3(), sheethead);
-                    fprintf(outfd, " -90 rotate (%s) show\n", sheethead);
-                }
-                else {
-                    /*
-                     * Filename on the left
-                     */
-                    fprintf(outfd, "%d %d moveto -90 rotate (%s) show\n",
-                            xbase1() + xwid1() + fsize/4, ytop4() - 3, fname);
-                    /*
-                     * Pagenumber on the right
-                     */
-                    fprintf(outfd, "90 rotate %d %d (Page %d) stringwidth pop ",
+#if 0
+                fprintf(outfd, "%d %d (%s) stringwidth pop 2 div add moveto",
+                               xbase1() + xwid1() + fsize/4,
+                               ybase3(), sheethead);
+                fprintf(outfd, " -90 rotate (%s) show\n", sheethead);
+#endif
+                /*
+                 * Filename/Header on the left
+                 */
+                fprintf(outfd, "%d %d moveto -90 rotate (%s) show\n",
+                            xbase1() + xwid1() + fsize/4, ytop4() - 3,
+			    *sheethead != '\0' ? sheethead : fname);
+                /*
+                 * Pagenumber on the right
+                 */
+                fprintf(outfd, "90 rotate %d %d (Page %d) stringwidth pop ",
                                    xbase1() + xwid1() + fsize/4,
                                    ybase1() + 3, ps_pagenum);
-                    fprintf(outfd, "add moveto -90 rotate (Page %d) show\n",
+                fprintf(outfd, "add moveto -90 rotate (Page %d) show\n",
                                    ps_pagenum);
-                }
                 break;
         }
         fprintf(outfd, "grestore\n");
